<?php

declare(strict_types=1);

namespace Drupal\solution;

use Drupal\Core\Field\EntityReferenceFieldItemList;
use Drupal\Core\TypedData\ComputedItemListTrait;

/**
 * Defines a field item list class for the 'releases_and_distributions' field.
 *
 * This computed field returns a list references to solution's releases and
 * standalone distributions. It's a read-only field.
 *
 * @todo This currently reuses the storage graph of the solution. This works
 *   only as intended for published solutions. In case the solution is not
 *   published, only the unpublished releases and distributions will be
 *   returned.
 *
 * @see https://citnet.tech.ec.europa.eu/CITnet/jira/browse/ISAICP-5736
 */
class SolutionReleasesAndDistributionsFieldItemList extends EntityReferenceFieldItemList {

  use ComputedItemListTrait;

  /**
   * {@inheritdoc}
   */
  protected function computeValue(): void {
    if ($this->getEntity()->id()) {
      foreach ($this->getReleasesAndDistributions() as $delta => $id) {
        $this->list[$delta] = $this->createItem($delta, ['target_id' => $id]);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function referencedEntities(): array {
    // Override the default value until we have all relevant entities as nodes.
    // @todo Remove this in ISAICP-8376.
    if ($this->isEmpty()) {
      return [];
    }

    // Collect the IDs of existing entities to load, and directly grab the
    // "autocreate" entities that are already populated in $item->entity.
    $target_entities = $ids = [];
    foreach ($this->list as $delta => $item) {
      if ($item->target_id !== NULL) {
        $ids[$delta] = $item->target_id;
      }
      elseif ($item->hasNewEntity()) {
        $target_entities[$delta] = $item->entity;
      }
    }

    // Load and add the existing entities.
    if ($ids) {
      $numeric_ids = [];
      $string_ids = [];
      foreach ($ids as $delta => $target_id) {
        if (is_numeric($target_id)) {
          $numeric_ids[$delta] = $target_id;
        }
        else {
          $string_ids[$delta] = $target_id;
        }
      }

      foreach (\Drupal::entityTypeManager()->getStorage('node')->loadMultiple($numeric_ids) as $delta => $referenced_entity) {
        $target_entities[$delta] = $referenced_entity;
      }

      foreach (\Drupal::entityTypeManager()->getStorage('rdf_entity')->loadMultiple($string_ids) as $delta => $referenced_entity) {
        $target_entities[$delta] = $referenced_entity;
      }

      // Ensure the returned array is ordered by deltas.
      ksort($target_entities);
    }

    return $target_entities;
  }

  /**
   * Returns a list of solution's releases and standalone distributions as IDs.
   *
   * @return string[]
   *   A list of solution's releases and standalone distributions as IDs.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   *   Thrown if the storage handler couldn't be loaded.
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the 'rdf_entity' entity type doesn't exist.
   */
  protected function getReleasesAndDistributions(): array {
    $entityTypeManager = \Drupal::entityTypeManager();
    /** @var \Drupal\sparql_entity_storage\SparqlEntityStorageInterface $storage */
    $storage = $entityTypeManager->getStorage('rdf_entity');
    /** @var \Drupal\solution\Entity\SolutionInterface $solution */
    $solution = $this->getEntity();
    $graph_id = $solution->get('graph')->target_id;

    // Retrieve all releases and distributions for this solution.
    $ids = array_values($storage
      ->getQuery()
      ->graphs([$graph_id])
      ->condition('rid', 'asset_release')
      ->condition('field_isr_is_version_of', $solution->id())
      ->sort('created', 'DESC')
      ->execute());

    // Retrieve also, all standalone distributions for this solution.
    return $ids + \Drupal::getContainer()->get('joinup_distribution.parent_distribution_resolver')->getParentsDistributionIds($solution->id());
  }

}
